@@ -16,7 +16,7 @@ use argmax::Command;
 use once_cell::sync::Lazy;
 use regex::Regex;
 
-use crate::exit_codes::ExitCode;
+use crate::exit_codes::{merge_exitcodes, ExitCode};
 
 use self::command::{execute_commands, handle_cmd_error};
 use self::input::{basename, dirname, remove_extension};
@@ -24,7 +24,7 @@ pub use self::job::{batch, job};
 use self::token::Token;
 
 /// Execution mode of the command
-#[derive(Debug, Clone, Copy, PartialEq)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub enum ExecutionMode {
     /// Command is executed for each search result
     OneByOne,
@@ -35,19 +35,17 @@ pub enum ExecutionMode {
 #[derive(Debug, Clone, PartialEq)]
 pub struct CommandSet {
     mode: ExecutionMode,
-    path_separator: Option<String>,
     commands: Vec<CommandTemplate>,
 }
 
 impl CommandSet {
-    pub fn new<I, S>(input: I, path_separator: Option<String>) -> Result<CommandSet>
+    pub fn new<I, S>(input: I) -> Result<CommandSet>
     where
         I: IntoIterator<Item = Vec<S>>,
         S: AsRef<str>,
     {
         Ok(CommandSet {
             mode: ExecutionMode::OneByOne,
-            path_separator,
             commands: input
                 .into_iter()
                 .map(CommandTemplate::new)
@@ -55,14 +53,13 @@ impl CommandSet {
         })
     }
 
-    pub fn new_batch<I, S>(input: I, path_separator: Option<String>) -> Result<CommandSet>
+    pub fn new_batch<I, S>(input: I) -> Result<CommandSet>
     where
         I: IntoIterator<Item = Vec<S>>,
         S: AsRef<str>,
     {
         Ok(CommandSet {
             mode: ExecutionMode::Batch,
-            path_separator,
             commands: input
                 .into_iter()
                 .map(|args| {
@@ -83,8 +80,13 @@ impl CommandSet {
         self.mode == ExecutionMode::Batch
     }
 
-    pub fn execute(&self, input: &Path, out_perm: Arc<Mutex<()>>, buffer_output: bool) -> ExitCode {
-        let path_separator = self.path_separator.as_deref();
+    pub fn execute(
+        &self,
+        input: &Path,
+        path_separator: Option<&str>,
+        out_perm: Arc<Mutex<()>>,
+        buffer_output: bool,
+    ) -> ExitCode {
         let commands = self
             .commands
             .iter()
@@ -92,12 +94,10 @@ impl CommandSet {
         execute_commands(commands, &out_perm, buffer_output)
     }
 
-    pub fn execute_batch<I>(&self, paths: I, limit: usize) -> ExitCode
+    pub fn execute_batch<I>(&self, paths: I, limit: usize, path_separator: Option<&str>) -> ExitCode
     where
         I: Iterator<Item = PathBuf>,
     {
-        let path_separator = self.path_separator.as_deref();
-
         let builders: io::Result<Vec<_>> = self
             .commands
             .iter()
@@ -120,7 +120,7 @@ impl CommandSet {
                     }
                 }
 
-                ExitCode::Success
+                merge_exitcodes(builders.iter().map(|b| b.exit_code()))
             }
             Err(e) => handle_cmd_error(None, e),
         }
@@ -136,6 +136,7 @@ struct CommandBuilder {
     cmd: Command,
     count: usize,
     limit: usize,
+    exit_code: ExitCode,
 }
 
 impl CommandBuilder {
@@ -147,7 +148,7 @@ impl CommandBuilder {
         for arg in &template.args {
             if arg.has_tokens() {
                 path_arg = Some(arg.clone());
-            } else if path_arg == None {
+            } else if path_arg.is_none() {
                 pre_args.push(arg.generate("", None));
             } else {
                 post_args.push(arg.generate("", None));
@@ -163,6 +164,7 @@ impl CommandBuilder {
             cmd,
             count: 0,
             limit,
+            exit_code: ExitCode::Success,
         })
     }
 
@@ -196,7 +198,9 @@ impl CommandBuilder {
     fn finish(&mut self) -> io::Result<()> {
         if self.count > 0 {
             self.cmd.try_args(&self.post_args)?;
-            self.cmd.status()?;
+            if !self.cmd.status()?.success() {
+                self.exit_code = ExitCode::GeneralError;
+            }
 
             self.cmd = Self::new_command(&self.pre_args)?;
             self.count = 0;
@@ -204,6 +208,10 @@ impl CommandBuilder {
 
         Ok(())
     }
+
+    fn exit_code(&self) -> ExitCode {
+        self.exit_code
+    }
 }
 
 /// Represents a template that is utilized to generate command strings.
@@ -293,9 +301,9 @@ impl CommandTemplate {
     /// Using the internal `args` field, and a supplied `input` variable, a `Command` will be
     /// build.
     fn generate(&self, input: &Path, path_separator: Option<&str>) -> io::Result<Command> {
-        let mut cmd = Command::new(self.args[0].generate(&input, path_separator));
+        let mut cmd = Command::new(self.args[0].generate(input, path_separator));
         for arg in &self.args[1..] {
-            cmd.try_arg(arg.generate(&input, path_separator))?;
+            cmd.try_arg(arg.generate(input, path_separator))?;
         }
         Ok(cmd)
     }
@@ -413,7 +421,7 @@ mod tests {
     #[test]
     fn tokens_with_placeholder() {
         assert_eq!(
-            CommandSet::new(vec![vec![&"echo", &"${SHELL}:"]], None).unwrap(),
+            CommandSet::new(vec![vec![&"echo", &"${SHELL}:"]]).unwrap(),
             CommandSet {
                 commands: vec![CommandTemplate {
                     args: vec![
@@ -423,7 +431,6 @@ mod tests {
                     ]
                 }],
                 mode: ExecutionMode::OneByOne,
-                path_separator: None,
             }
         );
     }
@@ -431,7 +438,7 @@ mod tests {
     #[test]
     fn tokens_with_no_extension() {
         assert_eq!(
-            CommandSet::new(vec![vec!["echo", "{.}"]], None).unwrap(),
+            CommandSet::new(vec![vec!["echo", "{.}"]]).unwrap(),
             CommandSet {
                 commands: vec![CommandTemplate {
                     args: vec![
@@ -440,7 +447,6 @@ mod tests {
                     ],
                 }],
                 mode: ExecutionMode::OneByOne,
-                path_separator: None,
             }
         );
     }
@@ -448,7 +454,7 @@ mod tests {
     #[test]
     fn tokens_with_basename() {
         assert_eq!(
-            CommandSet::new(vec![vec!["echo", "{/}"]], None).unwrap(),
+            CommandSet::new(vec![vec!["echo", "{/}"]]).unwrap(),
             CommandSet {
                 commands: vec![CommandTemplate {
                     args: vec![
@@ -457,7 +463,6 @@ mod tests {
                     ],
                 }],
                 mode: ExecutionMode::OneByOne,
-                path_separator: None,
             }
         );
     }
@@ -465,7 +470,7 @@ mod tests {
     #[test]
     fn tokens_with_parent() {
         assert_eq!(
-            CommandSet::new(vec![vec!["echo", "{//}"]], None).unwrap(),
+            CommandSet::new(vec![vec!["echo", "{//}"]]).unwrap(),
             CommandSet {
                 commands: vec![CommandTemplate {
                     args: vec![
@@ -474,7 +479,6 @@ mod tests {
                     ],
                 }],
                 mode: ExecutionMode::OneByOne,
-                path_separator: None,
             }
         );
     }
@@ -482,7 +486,7 @@ mod tests {
     #[test]
     fn tokens_with_basename_no_extension() {
         assert_eq!(
-            CommandSet::new(vec![vec!["echo", "{/.}"]], None).unwrap(),
+            CommandSet::new(vec![vec!["echo", "{/.}"]]).unwrap(),
             CommandSet {
                 commands: vec![CommandTemplate {
                     args: vec![
@@ -491,7 +495,6 @@ mod tests {
                     ],
                 }],
                 mode: ExecutionMode::OneByOne,
-                path_separator: None,
             }
         );
     }
@@ -499,7 +502,7 @@ mod tests {
     #[test]
     fn tokens_multiple() {
         assert_eq!(
-            CommandSet::new(vec![vec!["cp", "{}", "{/.}.ext"]], None).unwrap(),
+            CommandSet::new(vec![vec!["cp", "{}", "{/.}.ext"]]).unwrap(),
             CommandSet {
                 commands: vec![CommandTemplate {
                     args: vec![
@@ -512,7 +515,6 @@ mod tests {
                     ],
                 }],
                 mode: ExecutionMode::OneByOne,
-                path_separator: None,
             }
         );
     }
@@ -520,7 +522,7 @@ mod tests {
     #[test]
     fn tokens_single_batch() {
         assert_eq!(
-            CommandSet::new_batch(vec![vec!["echo", "{.}"]], None).unwrap(),
+            CommandSet::new_batch(vec![vec!["echo", "{.}"]]).unwrap(),
             CommandSet {
                 commands: vec![CommandTemplate {
                     args: vec![
@@ -529,14 +531,13 @@ mod tests {
                     ],
                 }],
                 mode: ExecutionMode::Batch,
-                path_separator: None,
             }
         );
     }
 
     #[test]
     fn tokens_multiple_batch() {
-        assert!(CommandSet::new_batch(vec![vec!["echo", "{.}", "{}"]], None).is_err());
+        assert!(CommandSet::new_batch(vec![vec!["echo", "{.}", "{}"]]).is_err());
     }
 
     #[test]
@@ -546,7 +547,7 @@ mod tests {
 
     #[test]
     fn command_set_no_args() {
-        assert!(CommandSet::new(vec![vec!["echo"], vec![]], None).is_err());
+        assert!(CommandSet::new(vec![vec!["echo"], vec![]]).is_err());
     }
 
     #[test]
