@@ -1,10 +1,11 @@
+use anyhow::anyhow;
 use once_cell::sync::Lazy;
 use regex::Regex;
 
 static SIZE_CAPTURES: Lazy<Regex> =
     Lazy::new(|| Regex::new(r"(?i)^([+-]?)(\d+)(b|[kmgt]i?b?)$").unwrap());
 
-#[derive(Clone, Copy, Debug, PartialEq)]
+#[derive(Clone, Copy, Debug, PartialEq, Eq)]
 pub enum SizeFilter {
     Max(u64),
     Min(u64),
@@ -24,7 +25,12 @@ const GIBI: u64 = MEBI * 1024;
 const TEBI: u64 = GIBI * 1024;
 
 impl SizeFilter {
-    pub fn from_string(s: &str) -> Option<Self> {
+    pub fn from_string(s: &str) -> anyhow::Result<Self> {
+        SizeFilter::parse_opt(s)
+            .ok_or_else(|| anyhow!("'{}' is not a valid size constraint. See 'fd --help'.", s))
+    }
+
+    fn parse_opt(s: &str) -> Option<Self> {
         if !SIZE_CAPTURES.is_match(s) {
             return None;
         }
@@ -165,7 +171,7 @@ mod tests {
                 #[test]
                 fn $name() {
                     let i = SizeFilter::from_string($value);
-                    assert!(i.is_none());
+                    assert!(i.is_err());
                 }
             )*
         };
