@@ -3,7 +3,7 @@ use chrono::{offset::TimeZone, DateTime, Local, NaiveDate};
 use std::time::SystemTime;
 
 /// Filter based on time ranges.
-#[derive(Debug, PartialEq)]
+#[derive(Debug, PartialEq, Eq)]
 pub enum TimeFilter {
     Before(SystemTime),
     After(SystemTime),
@@ -20,8 +20,8 @@ impl TimeFilter {
                     .ok()
                     .or_else(|| {
                         NaiveDate::parse_from_str(s, "%F")
-                            .map(|nd| nd.and_hms(0, 0, 0))
                             .ok()
+                            .and_then(|nd| nd.and_hms_opt(0, 0, 0))
                             .and_then(|ndt| Local.from_local_datetime(&ndt).single())
                     })
                     .or_else(|| Local.datetime_from_str(s, "%F %T").ok())
